// $Id$

package com.adventnet.servicedesk.common;

import java.net.URI;
import java.util.*;
import javax.mail.*;

import com.adventnet.asset.*;
import com.adventnet.ds.query.*;
import com.adventnet.servicedesk.security.Encoder;
import com.adventnet.servicedesk.utils.SDResourceBundle;
import com.manageengine.servicedesk.setup.util.HttpClientUtil;
import com.sun.mail.pop3.POP3SSLStore;
import com.sun.mail.pop3.POP3Store;
import com.sun.mail.imap.IMAPSSLStore;
import com.sun.mail.imap.IMAPStore;
import com.adventnet.persistence.DataObject;
import com.adventnet.persistence.Row;
import com.adventnet.servicedesk.utils.DataAccessUtil;
import com.adventnet.servicedesk.utils.DBUtilities;
import com.adventnet.servicedesk.utils.ResourcesUtil;
import com.adventnet.servicedesk.utils.ServiceDeskUtil;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.adventnet.servicedesk.util.CacheUtil;


import microsoft.exchange.webservices.data.core.ExchangeService;
import microsoft.exchange.webservices.data.core.WebProxy;
import microsoft.exchange.webservices.data.core.enumeration.misc.TraceFlags;
import microsoft.exchange.webservices.data.core.enumeration.property.WellKnownFolderName;
import microsoft.exchange.webservices.data.core.service.item.Item;
import microsoft.exchange.webservices.data.credential.ExchangeCredentials;
import microsoft.exchange.webservices.data.credential.WebCredentials;
import microsoft.exchange.webservices.data.credential.WebProxyCredentials;
import microsoft.exchange.webservices.data.misc.ITraceListener;
import microsoft.exchange.webservices.data.search.FindItemsResults;
import microsoft.exchange.webservices.data.search.ItemView;


public class MailUtilities
{
	Folder folder = null;
	Store store = null;
	private static MailUtilities mailutilities = null;
	private Logger logger = Logger.getLogger(MailUtilities.class.getName());

	private MailUtilities ()
	{
	}

	public static MailUtilities getInstance()
	{
		if(mailutilities == null)
			mailutilities = new MailUtilities();

		return mailutilities;
	}
	public HashMap tryFolderConnection(boolean closeResource) {
		HashMap connMap = new HashMap();
		int conRes=0;
		boolean isIMAP = false;
		String host = null;
		String username = null;
		String password = null;
		String mailbox = null;
		String protocol = null;
		String tls = "false"; //No I18N
		int port=0;
		try {
			host = (String) DBUtilities.getInstance().getResultObject("IncomingMailServer", null, "HOSTNAME"); //No I18N
			username = (String) DBUtilities.getInstance().getResultObject("IncomingMailServer", null, "USERNAME"); //No I18N
			password = (String) DBUtilities.getInstance().getResultObject("IncomingMailServer", null, "USERPWD"); //No I18N
			mailbox = (String) DBUtilities.getInstance().getResultObject("CommonMailProperties", null, "MAILBOX"); //No I18N
			Long portid = (Long) DBUtilities.getInstance().getResultObject("IncomingMailServer", null, "PORTID"); //No I18N
			logger.log(Level.INFO, "portid : " + portid); //No I18N
			Criteria ct = new Criteria(Column.getColumn("MailPortDetails", "PORTID"), portid, QueryConstants.EQUAL); //No I18N
			protocol = (String) DBUtilities.getInstance().getResultObject("MailPortDetails", ct, "EMAILTYPE"); //No I18N
			connMap.put("protocol", protocol);
			port = ((Long) DBUtilities.getInstance().getResultObject("MailPortDetails", ct, "PORT")).intValue(); //No I18N
			logger.log(Level.INFO, "protocol : " + protocol); //No I18N
			tls = (DBUtilities.getInstance().getResultObject("MailPortDetails", ct, "TLSENABLED")).toString(); //No I18N
			logger.log(Level.INFO, " tlsenabled : " + tls);
		} catch (Exception etn) {
			logger.log(Level.INFO, "Error in getting mail fetching information.");
			etn.printStackTrace();
			conRes = -10; //Error getting database values
			connMap.put("connresult", conRes);
			return connMap;
		}

		logger.log(Level.INFO, "host : " + host);
		logger.log(Level.INFO, "username : " + username);
		//System.out.println("password : "+password);
		logger.log(Level.INFO, "mailbox : " + mailbox);
		logger.log(Level.INFO, "protocol : " + protocol);
		logger.log(Level.INFO, "port : " + port);

		// Get system properties
		Properties props = System.getProperties();


		//For IMAP secured or POP3 secured

		String SSL_FACTORY = "javax.net.ssl.SSLSocketFactory"; //No I18N
		if (protocol.equalsIgnoreCase("pop3s")) {
			props.setProperty("mail.pop3.socketFactory.class", SSL_FACTORY); //No I18N
			props.setProperty("mail.pop3.socketFactory.fallback", "false"); //No I18N
			props.setProperty("mail.pop3.socketFactory.port", (new Integer(port)).toString()); //No I18N
			props.setProperty("mail.pop3.port", (new Integer(port)).toString()); //No I18N
		} else if (protocol.equalsIgnoreCase("imaps")) {
			props.setProperty("mail.imap.socketFactory.class", SSL_FACTORY); //No I18N
			props.setProperty("mail.imap.socketFactory.fallback", "false"); //No I18N
			props.setProperty("mail.imap.socketFactory.port", (new Integer(port)).toString()); //No I18N
			props.setProperty("mail.imap.port", (new Integer(port)).toString()); //No I18N
			if (tls.equalsIgnoreCase("true")) {
				props.setProperty("mail.imap.starttls.enable", "true"); //No I18N
			} else {
				props.setProperty("mail.imap.starttls.enable", "false"); //No I18N
			}

		}
		// To check if there are any additional mail properties to be included for mail processing..
		// get the additional mail properties based on mail protocol and add it to the properties field.
		try {
			checkAndAddMailProperties(protocol, props);
		} catch (Exception e) {
			logger.log(Level.SEVERE, "Exception while setting additional mail properties.  Hence not setting any additional mail properties.", e);
		}


		// Get session
		Session session = Session.getDefaultInstance(props, null);

		// Get the store
		try {
			try {
				Boolean isDebuggerEnabled = (Boolean) getCommonMailProperties().get("maildebug");//GlobalConfigUtil.getInstance().getGlobalConfigValue("status", "mail.debug"); //No I18N
				session.setDebug(isDebuggerEnabled);
			}
			catch (Exception e)
			{
				logger.log(Level.INFO, "Exception while fetching the maildebug value from commonmailproperties " + e.getMessage());
			}
			URLName url = new URLName(protocol, host, port, "", username, password);
			if (protocol.equalsIgnoreCase("pop3s")) {
				store = new POP3SSLStore(session, url);
				isIMAP = false;
			} else if (protocol.equalsIgnoreCase("imaps")) {
				store = new IMAPSSLStore(session, url);
				isIMAP = true;
			} else if (protocol.equalsIgnoreCase("pop3")) {
				store = new POP3Store(session, url);
				isIMAP = false;
			} else if (protocol.equalsIgnoreCase("imap")) {
				store = new IMAPStore(session, url);
				isIMAP = true;
			}

			logger.log(Level.INFO, "Obtained mail store from session.");

			store.connect(host, port, username, password);

		} catch (AuthenticationFailedException afe) {
			logger.log(Level.INFO, "Check if the username and password are correct");
			afe.printStackTrace();
			closeResources();
			conRes = -31;
			connMap.put("connresult", conRes);
			return connMap;
		} catch (MessagingException mexcep) {
			closeResources();
			logger.log(Level.INFO, "Exception when connecting to store.");
			conRes = -30; //Error connecting to store using the given details.
			mexcep.printStackTrace();
			//SD-15517 Incorrect error message is shown while saving the mail server details for POPS,IMAPS
			if (mexcep.toString().contains("PKIX")) {
				conRes = -33;
				connMap.put("connresult", conRes);
				return connMap;
			}
			if (mexcep.getCause() instanceof java.net.UnknownHostException)
			{
				conRes = -32;
			}

			if(mexcep.getCause() instanceof java.net.ConnectException)
			{
				conRes = -35;
			}

			connMap.put("connresult", conRes);
			return connMap;
		}
		// Get folder
		try
		{
			folder = store.getFolder(mailbox);
			/*if(folder.isOpen())
				folder.close(false);*/
			folder.open(Folder.READ_WRITE);
			conRes = 1;
			connMap.put("connresult", conRes);
			connMap.put("session", session);
			connMap.put("isIMAP", isIMAP);
			connMap.put("storeObj",store);
			connMap.put("mailbox",mailbox);
			connMap.put("folderObj",folder);

		}
		catch(MessagingException folderExcep)
		{
			logger.log(Level.INFO,"Exception when trying to open folder.");
			folderExcep.printStackTrace();
			closeResources();
			conRes = -40;
			connMap.put("connresult", conRes);
			return connMap;
		}
		//Resource has to be closed while doing test connection in mail server save operation.
		if(closeResource) {
			closeResources();
		}
		return connMap;
	}
	// This method is used to check if any additional mail properties needs to be added for the protocol passed.
	// If any mail properties is added and enabled for the protocol in 'MailProperties' table, will add the entries in
	// the passed properties object.
	public void checkAndAddMailProperties(String protocol, Properties props) throws Exception
	{
		// To check if there are any additional mail properties to be included for mail processing..
		// get the additional mail properties based on mail protocol and add it to the properties field.
		Iterator iter = ServiceDeskUtil.getInstance().getAdditionalMailProperties(protocol);
		if(iter != null)
		{
			while(iter.hasNext())
			{
				Row tempRow = (Row) iter.next();
				String propName = (String) tempRow.get("PROPNAME");  // No I18N
				String propValue = (String) tempRow.get("PROPVALUE");  // No I18N
				if(propName != null && propValue != null)
				{
					logger.log(Level.INFO, "Property to be added : {0}, value : {1}", new String [] {propName, propValue});
					props.setProperty(propName, propValue);
				}
			}
		}
	}

	public void closeResources()
	{
		// Close connection
		try
		{
			logger.log(Level.INFO,"Before closing folder in closeResources");

			if(folder != null && folder.isOpen())
			{
				logger.log(Level.INFO,"folder is open so closing...");
				folder.close(false);
			}

			logger.log(Level.INFO,"before closing store");

			if(store != null && store.isConnected())
			{
				logger.log(Level.INFO,"store is connected so closing...");
				store.close();
			}
		}
		catch(Exception xcep)
		{
			logger.log(Level.INFO,"Exception when closing folder/store");
			xcep.printStackTrace();
		}
	}

	/****
	*
	* Method to tell if SMTP Authentication is required for OutGoing mails
	*
	*****/

	public boolean isSMTPAuthRequired() throws Exception
	{
		//Searches for the Category : SMTPOutgoingMail, Parameter:  REQUIRE_AUTHENTICATION in GlobalConfig Table

		String data = "false"; // No I18n
		DataObject globalDO = ServiceDeskUtil.getInstance().getGlobalConfigCache();

		Criteria ct1 = new Criteria(Column.getColumn(GLOBALCONFIG.TABLE,GLOBALCONFIG.CATEGORY), "SMTPOutgoingMail", QueryConstants.EQUAL); //No I18N
		Criteria ct2 = new Criteria(Column.getColumn(GLOBALCONFIG.TABLE,GLOBALCONFIG.PARAMETER), "REQUIRE_AUTHENTICATION", QueryConstants.EQUAL); //No I18N
		ct1 = ct1.and(ct2);

		Object obj = globalDO.getValue(GLOBALCONFIG.TABLE, GLOBALCONFIG.PARAMVALUE, ct1);
		if(obj != null)
		{
			data = obj.toString();
		}

		logger.log(Level.INFO," Is SMTP Authentication Required : {0}",data);

		return Boolean.parseBoolean(data);
	}


	/****
	*
	* To enable SMTPAuthentication as Required
	*
	*****/
	public void setSMTPAuthentication(boolean isRequired) throws Exception
	{

		try
		{
			String gconCategory = "SMTPOutgoingMail"; //No I18N
			String gconParameter = "REQUIRE_AUTHENTICATION"; //No I18N

            String data = isRequired + "";


			String table = GLOBALCONFIG.TABLE;
        	SelectQuery sq = new SelectQueryImpl(Table.getTable(table));
        	sq.addSelectColumn(Column.getColumn(table, "*"));

			Criteria ct1 = new Criteria(Column.getColumn(GLOBALCONFIG.TABLE,GLOBALCONFIG.CATEGORY), gconCategory, QueryConstants.EQUAL);
			Criteria ct2 = new Criteria(Column.getColumn(GLOBALCONFIG.TABLE,GLOBALCONFIG.PARAMETER), gconParameter, QueryConstants.EQUAL);
			ct1 = ct1.and(ct2);

			sq.setCriteria(ct1);

	        DataObject dObject = DataAccessUtil.getInstance().get(sq);
			if(dObject.isEmpty())
			{
				//Add new row to the GlobalConfig
				Row row = new Row(table);
            	row.set(GLOBALCONFIG.CATEGORY, gconCategory);
            	row.set(GLOBALCONFIG.PARAMETER, gconParameter);
            	row.set(GLOBALCONFIG.PARAMVALUE, data);
            	dObject.addRow(row);
            	ResourcesUtil.getInstance().add(dObject);
			}
			else
			{
	            Row row = dObject.getFirstRow(table);
               	row.set(GLOBALCONFIG.PARAMVALUE, data);
               	dObject.updateRow(row);
               	ResourcesUtil.getInstance().update(dObject);

				//Edit Existing Table
				ResourcesUtil.getInstance().update(dObject);
			}
       		//boolean updateState = DBUtilities.getInstance().updateColumnStatus(GLOBALCONFIG.TABLE,GLOBALCONFIG.PARAMVALUE,isRequired,ct1);
		}
		catch (Exception e)
		{
			logger.log(Level.SEVERE,"Exception occured while setting SMTPAuthentication");
			e.printStackTrace();
			throw e;
		}

	}

        /**
         * Returns the Incoming Mail Server details as a DataObject.
         *
         * @return
         * @throws Exception
         */
    public DataObject getIncomingMailServerDetails() throws Exception {
        DataObject mailDO = (DataObject) CacheUtil.getFromCache(CacheUtil.INCOMING_MAIL_SERVER); // No I18N
        if (mailDO == null) {
            mailDO = DataAccessUtil.getInstance().get(INCOMINGMAILSERVER.TABLE, (Criteria) null);
            List tableNames = new ArrayList();
            tableNames.add(INCOMINGMAILSERVER.TABLE);
            CacheUtil.addToCache(CacheUtil.INCOMING_MAIL_SERVER, mailDO, tableNames);
        }
        return mailDO;
    }

    /**
     * Returns the rows in the MailPortDetails table as a DataObject
     *
     * @return
     * @throws Exception
     */
    public DataObject getMailPortDetails() throws Exception {
        DataObject portDO = (DataObject) CacheUtil.getFromCache(CacheUtil.MAIL_PORT_DETAILS);
        if (portDO == null) {
        	portDO = DataAccessUtil.getInstance().get(MAILPORTDETAILS.TABLE, (Criteria) null);
        	List tableNames = new ArrayList();
			    //Issue - 56143 fix
        	tableNames.add(MAILPORTDETAILS.TABLE);
            CacheUtil.addToCache(CacheUtil.MAIL_PORT_DETAILS, portDO, tableNames);
        }
        return portDO;
    }

    /**
     * Returns the Outgoing Mail Server details as a DataObject.
     *
     * @return
     * @throws Exception
     */
    public DataObject getOutgoingMailServerDetails() throws Exception
    {
    	DataObject mailDO = (DataObject) CacheUtil.getFromCache(CacheUtil.OUTGOING_MAIL_SERVER); // No I18N
    	if (mailDO == null)
    	{
    		mailDO = DataAccessUtil.getInstance().get(OUTGOINGMAILSERVER.TABLE, (Criteria) null);
    		List tableNames = new ArrayList();
    		tableNames.add(OUTGOINGMAILSERVER.TABLE);
    		CacheUtil.addToCache(CacheUtil.OUTGOING_MAIL_SERVER, mailDO, tableNames);
    	}
    	return mailDO;
    }

    /**
     * Returns the Outgoing Mail Aliases details as a DataObject.
     *
     * @return
     * @throws Exception
     */
    public DataObject getOutgoingMailAliases() throws Exception
    {
    	DataObject aliasDO = (DataObject) CacheUtil.getFromCache(CacheUtil.OUTGOING_MAIL_ALIASES); // No I18N
    	if (aliasDO == null)
    	{
    		aliasDO = DataAccessUtil.getInstance().get(OUTGOINGMAILALIASES.TABLE, (Criteria) null);
    		List tableNames = new ArrayList(1);
    		tableNames.add(OUTGOINGMAILALIASES.TABLE);
    		CacheUtil.addToCache(CacheUtil.OUTGOING_MAIL_ALIASES, aliasDO, tableNames);
    	}
    	return aliasDO;
    }

	/**
	 * Whenever a notification is sent from SDP, this method would be called. The message Id and the sent time of the notification mail will be put in the cache.
         *
	 * @param messageId
	 * @param sentTime
	 * @throws Exception
	 */
	public void addToMessageIdCache(String messageId, Long sentTime) throws Exception {
            
            synchronized(CacheUtil.MESSAGE_ID_CACHE){
                Integer cacheSize = CacheUtil.getLRUCacheSize(CacheUtil.MESSAGE_ID_CACHE);
                if(cacheSize == null){
                    LinkedHashMap cacheObj = getMessageIdCache(); //if cache is empty, then this will create the cache from db. 
	}
		CacheUtil.addToLRUCache(CacheUtil.MESSAGE_ID_CACHE, messageId, sentTime, null);
	}
	}

	/**
	 * Returns the least time that is available in the cache.
         *
	 * @param messageId
	 * @param sentTime
	 * @throws Exception
	 */
	public Long fetchLeastTimeFromCache() throws Exception {
            
		LinkedHashMap cachedObject = getMessageIdCache();
		Long cachedSentTime = (Long) CacheUtil.getFromCache(CacheUtil.MESSAGEID_LEAST_TIME);// No I18N
		
                    if(cachedObject != null){
                        
		if(!cachedObject.containsValue(cachedSentTime) && !cachedObject.isEmpty()) {
                        cachedSentTime = null;
				Iterator iter = cachedObject.entrySet().iterator();
			while(iter.hasNext()) {
                                Map.Entry e = (Map.Entry) iter.next();
					Long time = (Long) e.getValue();
	                                if(cachedSentTime == null) {
	                                    cachedSentTime = time;
	                                }
	                                else if(cachedSentTime > time) {
	                                    cachedSentTime = time;
	                                }
				}
			CacheUtil.addToCache(CacheUtil.MESSAGEID_LEAST_TIME, cachedSentTime, null);
			
		}
                }
		return cachedSentTime;
	}

        /**
         * Will return whether the  passed messageId is one of our own notification mail or not. Initially, the messageId will be checked whether it is present
         * in the LRUMap cache or not. If it is present, then it means that the message was sent by us. So, true will be returned. If the message Id is not present,
         * then the least time that is available in the cache will be fetched and compared. If the sent time is greater than the cached least time, then it means that
         * the message was not sent by us, If it is less, then it could be that, we do not have the message Id in the cache. so a query will be sent to the
         * database to check whether the message Id was sent by us or not.
         *
         * @param messageId
         * @param sentTime
         * @return
         * @throws Exception
         */
	public boolean isNotificationMail(String messageId, Long sentTime) throws Exception {
		boolean isNotificationMail = false;
		if (messageId != null) {
			
                        if(CacheUtil.checkLRUCacheContainsKey(CacheUtil.MESSAGE_ID_CACHE, messageId))
                        {
                           
                            return true;
                        }

			// Message Id is not present in the cache. Now got to check the least time.
			Long cachedSentTime = fetchLeastTimeFromCache();
                        if(cachedSentTime == null) {
                            cachedSentTime = 0L;
                        }
			if (sentTime > cachedSentTime) {
				// If the mail sent time is greater than the cached least time, then it means that the message has to be a new one, else we would have found a match.
				isNotificationMail = false;
			}
			else if (sentTime < cachedSentTime) {
				// Means that we do not have the message id in cache. So have to do a DB check, just in case.
				try {
					DataObject notDO = DataAccessUtil.getInstance().get(NOTIFICATION.TABLE, new Criteria(Column.getColumn(NOTIFICATION.TABLE, NOTIFICATION.MESSAGEID), messageId, QueryConstants.EQUAL, false));
					if (!notDO.isEmpty() && notDO.containsTable(NOTIFICATION.TABLE)) {
						isNotificationMail = true;
					}
				}
				catch (Exception e) {
					e.printStackTrace();
					logger.log(Level.INFO, " Exception while trying to check isNotificationMail.");
				}
			}
		}
		return isNotificationMail;
	}

	/**
	 * Returns a LRUMap of the Message ID to Sent Time cache. The number of objects to be cached is configurable and can be changed by modifying the MESSAGE_ID_CACHE
         * variable in GlobalConfig. By default, the value is 1000. When the LRUMap, is not present in the database, the Notification table will be queried for the
         * message Ids with limit = cacheCount and sorted by Notification ID. The fetched ID and time will be stored to a LRUMap and returned.
         *
	 * @param messageId
	 * @param sentTime
	 * @throws Exception
	 */
	
        
        public LinkedHashMap getMessageIdCache() throws Exception {
            
            Integer cacheSize = CacheUtil.getLRUCacheSize(CacheUtil.MESSAGE_ID_CACHE);
            
            if(cacheSize == null){
                synchronized(CacheUtil.MESSAGE_ID_CACHE){
                    cacheSize = CacheUtil.getLRUCacheSize(CacheUtil.MESSAGE_ID_CACHE);
                    if(cacheSize == null){
                int cacheCount = 1000;
                cacheCount = CacheUtil.getLRUCacheMaxSize("MESSAGE_ID_CACHE"); //No I18N
                
                SelectQueryImpl sqlImpl = new SelectQueryImpl(new Table(NOTIFICATION.TABLE));
                sqlImpl.addSelectColumn(Column.getColumn(NOTIFICATION.TABLE, NOTIFICATION.NOTIFICATIONID));
                sqlImpl.addSelectColumn(Column.getColumn(NOTIFICATION.TABLE, NOTIFICATION.MESSAGEID));
                sqlImpl.addSelectColumn(Column.getColumn(NOTIFICATION.TABLE, NOTIFICATION.NOTIFICATIONDATE));
                sqlImpl.setRange(new Range(0, cacheCount));
                sqlImpl.addSortColumn(new SortColumn(Column.getColumn(NOTIFICATION.TABLE, NOTIFICATION.NOTIFICATIONID), false));
                DataObject messageDO = DataAccessUtil.getInstance().get(sqlImpl);
                
                Iterator iter = messageDO.getRows(NOTIFICATION.TABLE);
                while(iter.hasNext()){
                        Row messageRow = (Row) iter.next();
                        CacheUtil.addToLRUCache(CacheUtil.MESSAGE_ID_CACHE, (String) messageRow.get(NOTIFICATION.MESSAGEID), (Long) messageRow.get(NOTIFICATION.NOTIFICATIONDATE) , null);
                }   
                }
            }
            }
            LinkedHashMap cachedObject = new LinkedHashMap(); 
            cachedObject = CacheUtil.getLRUCacheAsLinkedHashMap(CacheUtil.MESSAGE_ID_CACHE); 
            //logger.log(Level.INFO,"cached Object  " + cachedObject.toString()) ;
            return cachedObject;
            
            
        }

        // variable used to synchronize the cache creation..
        private static Integer syncInt = new Integer(1);

        /** To return the entries added as Incoming mail alias.
         *
         * @return - DataObject containing the entries in the IncomingMailAliases table
         * @throws Exception
         */
        public DataObject getIncomingMailAliases() throws Exception
        {
            // get the incoming mail alias
            // Todo : To have the cache name from CacheUtil static variable
            String cacheName = CacheUtil.INCOMING_MAIL_ALIASES;
            DataObject dobj = (DataObject) CacheUtil.getFromCache(cacheName);
            if(dobj == null)
            {
                synchronized(syncInt)
                {
                    // the double checking is included to avoid synchornized check for every call..
                    dobj = (DataObject) CacheUtil.getFromCache(cacheName);
                    if(dobj == null)
                    {
                        Criteria crit = null;
                        dobj = DataAccessUtil.getInstance().get(INCOMINGMAILALIASES.TABLE, crit);
                        ArrayList list2 = new ArrayList(1);
                        String tableName = INCOMINGMAILALIASES.TABLE;
                        list2.add(tableName);
                        CacheUtil.addToCache(cacheName, dobj, list2);
                    }// if(list == null)
                }// end (synchronized)
            }// if(list == null)
            return dobj;
        }
	public HashMap getCommonMailProperties()throws Exception
	{
		HashMap propsMap = new HashMap();
		ArrayList tableNames = new ArrayList();
		tableNames.add(COMMONMAILPROPERTIES.TABLE);
		DataObject propsDO = (DataObject) CacheUtil.getFromCache(CacheUtil.COMMONMAILPROPERTIES, tableNames, true);
		if(propsDO == null) {
			propsDO = ResourcesUtil.getInstance().get(COMMONMAILPROPERTIES.TABLE, (Criteria) null);
			CacheUtil.addToCache(CacheUtil.COMMONMAILPROPERTIES, propsDO, tableNames);
		}
		Row row = propsDO.getRow(COMMONMAILPROPERTIES.TABLE, (Criteria) null);
		propsMap.put("fetchinterval", row.get(COMMONMAILPROPERTIES.MAILFETCHINTERVAL));
		propsMap.put("mailbox", row.get(COMMONMAILPROPERTIES.MAILBOX));
		propsMap.put("disablenewrequest", row.get(COMMONMAILPROPERTIES.DISABLENEWREQUEST));
		propsMap.put("exceptionlist", row.get(COMMONMAILPROPERTIES.EXCEPTIONLIST));
		propsMap.put("maildebug", row.get(COMMONMAILPROPERTIES.MAILDEBUG));
		return propsMap;
	}
	public String getSelectedIncomingServer()throws Exception
	{
		String incomingServer = null;
		ArrayList tableNames = new ArrayList();
		tableNames.add(INCOMINGMAILSERVER.TABLE);
		tableNames.add(INCOMINGEWSSERVER.TABLE);
		incomingServer = (String) CacheUtil.getFromCache(CacheUtil.SELECTED_INCOMINGSERVER, tableNames, true);
		if(incomingServer == null)
		{
			//Getting javamail details
			Criteria cri = new Criteria(new Column(INCOMINGMAILSERVER.TABLE,INCOMINGMAILSERVER.ISENABLED),true,QueryConstants.EQUAL);
			DataObject mailAPIDO =  DataAccessUtil.getInstance().get(INCOMINGMAILSERVER.TABLE,cri);
			if(!mailAPIDO.isEmpty())
			{
				incomingServer = "javamail";//NO I18N
			}
			//Getting EWSAPI details
			cri = new Criteria(new Column(INCOMINGEWSSERVER.TABLE,INCOMINGEWSSERVER.ISENABLED),true,QueryConstants.EQUAL);
			DataObject ewsAPIDO =  DataAccessUtil.getInstance().get(INCOMINGEWSSERVER.TABLE,cri);
			if(!ewsAPIDO.isEmpty())
			{
				incomingServer = "ews";//NO I18N
			}
			CacheUtil.addToCache(CacheUtil.SELECTED_INCOMINGSERVER, incomingServer, tableNames);
		}
		return incomingServer;
	}
	public String getSelectedOutgoingServer()throws Exception
	{
		String outgoingServer = null;
		ArrayList tableNames = new ArrayList();
		tableNames.add(OUTGOINGMAILSERVER.TABLE);
		tableNames.add(OUTGOINGEWSSERVER.TABLE);
		outgoingServer = (String) CacheUtil.getFromCache(CacheUtil.SELECTED_OUTGOINGSERVER, tableNames, true);
		if(outgoingServer == null)
		{
			//Getting javamail details
			Criteria cri = new Criteria(new Column(OUTGOINGMAILSERVER.TABLE,OUTGOINGMAILSERVER.ISENABLED),true,QueryConstants.EQUAL);
			DataObject mailAPIDO =  DataAccessUtil.getInstance().get(OUTGOINGMAILSERVER.TABLE,cri);
			if(!mailAPIDO.isEmpty())
			{
				outgoingServer = "javamail";//NO I18N
			}
			//Getting EWSAPI details
			cri = new Criteria(new Column(OUTGOINGEWSSERVER.TABLE,OUTGOINGEWSSERVER.ISENABLED),true,QueryConstants.EQUAL);
			DataObject ewsAPIDO =  DataAccessUtil.getInstance().get(OUTGOINGEWSSERVER.TABLE,cri);
			if(!ewsAPIDO.isEmpty())
			{
				outgoingServer = "ews";//NO I18N
			}
			CacheUtil.addToCache(CacheUtil.SELECTED_OUTGOINGSERVER, outgoingServer, tableNames);
		}
		return outgoingServer;
	}
	public void setCommonMailProperties(String interval, String disablenewRequest, String exceptionList, String mailDebug)throws Exception {

		DataObject dobj = DataAccessUtil.getInstance().get(COMMONMAILPROPERTIES.TABLE, (Criteria)null);
		if(!dobj.isEmpty()) {
			Row propRow = (Row) dobj.getRow(COMMONMAILPROPERTIES.TABLE, (Criteria) null);
			propRow.set(COMMONMAILPROPERTIES.MAILDEBUG, Boolean.valueOf(mailDebug));
			propRow.set(COMMONMAILPROPERTIES.MAILFETCHINTERVAL, new Long(interval));
			propRow.set(COMMONMAILPROPERTIES.DISABLENEWREQUEST, Boolean.valueOf(disablenewRequest));
			propRow.set(COMMONMAILPROPERTIES.EXCEPTIONLIST, exceptionList);
			dobj.updateRow(propRow);
			ResourcesUtil.getInstance().update(dobj);
		}
	}
	public int testIncomingEWSServer()
	{
		HashMap connMap = tryIncomingEWSServer();
		return (Integer)connMap.get("connresult");
	}
	public HashMap tryIncomingEWSServer()
	{
		HashMap connMap = new HashMap();
		int conRes = 0;
		ExchangeService service = new ExchangeService();
		try {
			DataObject ewsDO = DataAccessUtil.getInstance().get(INCOMINGEWSSERVER.TABLE,(Criteria)null);
			Row incomingRow = (Row)ewsDO.getRow(INCOMINGEWSSERVER.TABLE);
			String ewsURL = (String)incomingRow.get(INCOMINGEWSSERVER.SERVERURL);
			String ewsUsername = (String)incomingRow.get(INCOMINGEWSSERVER.USERNAME);
			String ewsPassword = (String)incomingRow.get(INCOMINGEWSSERVER.USERPWD);
			logger.log(Level.INFO,"ewsurl::"+ewsURL);
			service.setUrl(new URI(ewsURL));
			try
			{
				logger.log(Level.INFO, "!@#!#@ Going to set proxy for ews service");
				logger.log(Level.INFO, "!@#!@# fetch proxyDetails ::");
				HashMap<String, String> proxyDetails = HttpClientUtil.fetchProxyDetails();
				logger.log(Level.INFO, "!@#!@# " + proxyDetails);
				String proxyHost = proxyDetails.get("proxyHost");
				String proxyPort = proxyDetails.get("proxyPort");
				String proxyUsername = proxyDetails.get("proxyUsername");
				String proxyPassword = proxyDetails.get("proxyPassword");
				WebProxyCredentials webProxyCredentials = null;
				if (!"".equals(proxyUsername) || !"".equals(proxyPassword)) {
					int domainIndex = proxyUsername.contains("\\") ? proxyUsername.lastIndexOf("\\") : -1;
					String domain = null;
					if(domainIndex != -1)
					{
						domain = proxyUsername.substring(0, domainIndex);
						proxyUsername = proxyUsername.substring(domainIndex + 1);
					}
					if (!"".equals(proxyUsername)) {
						proxyPassword = "".equals(proxyPassword) ? null : proxyPassword;
						logger.log(Level.INFO, "!@#!@# Going to set credentials for proxy - username ::"+proxyUsername+" :: password ::"+proxyPassword+" :: domain ::"+domain);
						webProxyCredentials = new WebProxyCredentials(proxyUsername, proxyPassword, domain);
					}
				}
				if (!"".equals(proxyHost) && !"".equals(proxyPort)) {
					int proxyPortInt = Integer.parseInt(proxyPort);
					logger.log(Level.INFO, "!@#!@# Going to set hostname and port for proxy - hostname ::"+proxyHost+" :: port ::"+proxyPortInt);
					WebProxy webProxy = webProxyCredentials == null ? new WebProxy(proxyHost, proxyPortInt) : new WebProxy(proxyHost, proxyPortInt, webProxyCredentials);
					service.setWebProxy(webProxy);
				}
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
				logger.log(Level.INFO, "!@#!@# Exception while setting proxy for ews");
			}
			//SD-77145
			ExchangeCredentials credentials;
			int domainIndex = ewsUsername.indexOf("\\");
			if(domainIndex != -1)
			{
				String domain = ewsUsername.substring(0, domainIndex);
				ewsUsername = ewsUsername.substring(domainIndex +1);
				credentials = new WebCredentials(ewsUsername, ewsPassword, domain);
			}
			else
			{
				credentials = new WebCredentials(ewsUsername, ewsPassword);
			}
			service.setCredentials(credentials);
			Boolean isDebuggerEnabled = (Boolean)getCommonMailProperties().get("maildebug");
			if(isDebuggerEnabled) {
				service.setTraceEnabled(true);
				service.setTraceFlags(EnumSet.allOf(TraceFlags.class));
				service.setTraceListener(new ITraceListener() {
					public void trace(String traceType, String traceMessage) {
						logger.log(Level.INFO, "Type:" + traceType + " Message:" + traceMessage);
					}
				});
			}
			FindItemsResults<Item> findResults = service.findItems(WellKnownFolderName.Inbox, new ItemView(1));
			findResults.getTotalCount();
			conRes = 1;
			connMap.put("connresult",conRes);
			connMap.put("service",service);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			String rootCause = getRootException(ex).toString();
			if(rootCause.contains("(401)Unauthorized")) {
				conRes = 301;
			}
			else if(rootCause.contains("(404)Not Found"))
			{
				conRes = 302;
			}
			else if(rootCause.contains("java.net.UnknownHostException")) {
				conRes = 303;
			}
			else if(rootCause.contains("java.net.NoRouteToHostException")) {
				conRes = 304;
			}
			else if(rootCause.contains("NullPointerException") && ex.getMessage().contains("The request failed. null"))
			{
				conRes = 305;
			}
			else
			{
				conRes = 400;
			}
			connMap.put("connresult",conRes);
		}

		return connMap;
	}
	public int testIncomingServer()
	{
		HashMap connMap = tryFolderConnection(true);
		int connresult = (Integer)connMap.get("connresult");
		return connresult;
	}
	/**
	 * To check if authentication is necessary for outgoing mails.
	 * Checks if authentication is necessary and returns the approrpriate SMTPAuthenticator
	 * Also when authentication is necessary the "mail.smtp.auth" will be set as "true" in the input Properties arg.
	 *
	 * If authentication is not required then a null object will be returned
	 * @param props
	 * @return
	 * @throws Exception
	 */
	public SMTPAuthenticator getSMTPAuthenticator(Properties props) throws Exception
	{
		SMTPAuthenticator authenticator = null;
		if (MailUtilities.getInstance().isSMTPAuthRequired())
		{
			props.put("mail.smtp.auth", "true");//No I18N
			//To generate Outgoing SelectQuery
			String tableName = OUTGOINGMAILSERVER.TABLE;
			String tableName1 = SMTPAUTHLOGININFO.TABLE;
			String tableName2 = PASSWORDINFO.TABLE;

			SelectQuery sq = new SelectQueryImpl(Table.getTable(tableName));
			sq.addSelectColumn(Column.getColumn(tableName, OUTGOINGMAILSERVER.SERVERID));
			sq.addSelectColumn(Column.getColumn(tableName, OUTGOINGMAILSERVER.HOSTNAME));
			sq.addSelectColumn(Column.getColumn(tableName, OUTGOINGMAILSERVER.ALTERNATEHOSTNAME));
			sq.addSelectColumn(Column.getColumn(tableName, OUTGOINGMAILSERVER.PORTID));
			sq.addSelectColumn(Column.getColumn(tableName1, SMTPAUTHLOGININFO.SERVERID));
			sq.addSelectColumn(Column.getColumn(tableName1, SMTPAUTHLOGININFO.LOGINNAME));
			sq.addSelectColumn(Column.getColumn(tableName1, SMTPAUTHLOGININFO.PASSWORDID));
			sq.addSelectColumn(Column.getColumn(tableName2, PASSWORDINFO.PASSWORDID));
			sq.addSelectColumn(Column.getColumn(tableName2, PASSWORDINFO.DOMAINPASSWORD));

			String baseColumns[] = {OUTGOINGMAILSERVER.SERVERID};
			Join j1 = new Join(tableName, tableName1, baseColumns, baseColumns, Join.LEFT_JOIN);
			sq.addJoin(j1);

			baseColumns = new String[]{PASSWORDINFO.PASSWORDID};
			Join j2 = new Join(tableName1, tableName2, baseColumns, baseColumns, Join.LEFT_JOIN);
			sq.addJoin(j2);

			DataObject outgoingDO = DataAccessUtil.getInstance().get(sq);
			String userName = null;
			String passWord = null;
			if (outgoingDO.containsTable(SMTPAUTHLOGININFO.TABLE))
			{
				userName = (String) outgoingDO.getFirstValue(SMTPAUTHLOGININFO.TABLE, SMTPAUTHLOGININFO.LOGINNAME);
			}
			if (outgoingDO.containsTable(PASSWORDINFO.TABLE))
			{
				passWord = (String) outgoingDO.getFirstValue(PASSWORDINFO.TABLE, PASSWORDINFO.DOMAINPASSWORD);
			}
			authenticator = new SMTPAuthenticator(userName, passWord);
		}
		else {    // Fix for issue: If SMTP auth is disabled from UI, need an application restart for it to take effect
			props.put("mail.smtp.auth", "false");//No I18N
		}
		return authenticator;
	}

	/**
	 * This method is used to get the pendingMailCount value from CommonMailProperties table.
	 * @return
	 */

	public Long getPendingMailCount()
	{
		Long pendingMailCount = null;
		try
		{
			String mailServer = getSelectedIncomingServer();
			if("javamail".equals(mailServer))
			{
				HashMap connMap = tryFolderConnection(false);
				int conRes = (Integer) connMap.get("connresult");
				if(conRes == 1)
				{
					Folder folder = (Folder) connMap.get("folderObj");
					try
					{
						int count = folder.getMessageCount();
						pendingMailCount = new Long(count);
					}
					catch(Exception e)
					{
						e.printStackTrace();
					}
					finally
					{
						closeResources();
					}
				}
			}
			else
			{
				try
				{
					HashMap ewsMap = tryIncomingEWSServer();
					int connRes = (Integer) ewsMap.get("connresult");
					if(connRes == 1)
					{
						ExchangeService service = (ExchangeService) ewsMap.get("service");
						int offset = 0;
						int number_of_emails_to_fetch = 2;
						FindItemsResults<Item> inboxEmailResults;
						ItemView view = new ItemView(number_of_emails_to_fetch, offset);
						inboxEmailResults = service.findItems(WellKnownFolderName.Inbox, view);
						int total_msg_count = inboxEmailResults.getTotalCount();
						pendingMailCount = new Long(total_msg_count);
					}

				}
				catch(Exception ex)
				{
					ex.printStackTrace();
				}
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}

		return pendingMailCount;
	}


	public static Throwable getRootException(Throwable exception){
		Throwable rootException=exception;
		while(rootException.getCause()!=null){
			rootException = rootException.getCause();
		}
		return rootException;
	}


	public String getMailErrorMessage(int errorCode)
	{
		try {
			SDResourceBundle bnd = new SDResourceBundle(ServiceDeskUtil.getInstance().getUserId());
			switch (errorCode) {
				case -10: {
					return bnd.getString("sdp.admin.testmail.incoming.10");
				}

				case -31: {
					return bnd.getString("sdp.admin.testmail.incoming.31");
				}

				case -32: {
					return bnd.getString("sdp.admin.testmail.incoming.32");
				}

				case -33: {
					return bnd.getString("sdp.admin.testmail.incoming.33");
				}

				case -35: {
					return bnd.getString("sdp.admin.testmail.incoming.35");
				}

				case -40: {
					return bnd.getString("sdp.admin.testmail.incoming.40");
				}

				case 301 :
					return bnd.getString("sdp.admin.testmail.incoming.31");

				case 302 :
					return bnd.getString("sdp.admin.testmail.incoming.302");

				case 303 :
					return bnd.getString("sdp.admin.testmail.incoming.32");

				case 304 :
					return bnd.getString("sdp.admin.testmail.incoming.35");

				case 305 :
					return bnd.getString("sdp.admin.testmail.incoming.302");

				case 400 :
					return bnd.getString("sdp.admin.testmail.incoming.30");

				default:
					return bnd.getString("sdp.admin.testmail.error");
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return "Unknown Error occurred. Please refer to logs"; //NO I18N
		}
	}
}
